from channels.generic.websocket import WebsocketConsumer
from asgiref.sync import async_to_sync
from .models import ChatMemberModel, ChatMessageModel
from profiles.models import BlockModel

import time
import json

class ChatConsumer(WebsocketConsumer):

	def connect(self):

		user = self.scope["user"]
		if (user.is_anonymous or not user.is_authenticated):
			return

		channel_id : int = int(self.scope['url_route']['kwargs']['chat_id'])

		self.room_group_name = f'chat{channel_id}'

		if ChatMemberModel.objects.filter(member_id=user.pk, channel_id=int(channel_id)).count() != 1:
			return

		self.room_group_name = f'chat{channel_id}'

		if (self.channel_layer == None):
			return

		async_to_sync(self.channel_layer.group_add)(
			self.room_group_name,
			self.channel_name
		)

		self.accept()


	def receive(self, text_data=None, bytes_data=None):

		if text_data == None:
			return

		user = self.scope["user"]
		if (user.is_anonymous or not user.is_authenticated):
			return

		text_data_json: dict = json.loads(text_data)
		
		message = text_data_json.get('message')
		if (message is None):
			return
		
		receivers_id = text_data_json.get('receivers_id')
		if (receivers_id is None):
			return

		channel_id : int = int(self.scope['url_route']['kwargs']['chat_id'])

		if ChatMemberModel.objects.filter(member_id = user.pk, channel_id = channel_id).count() != 1:
			return

		if (self.channel_layer == None):
			return

		message_time: int = int(time.time() * 1000)
		
		if (len(receivers_id) == 1 and 
			BlockModel.objects.filter(blocker=user.pk, blocked=receivers_id[0]) or
			BlockModel.objects.filter(blocker=receivers_id[0], blocked=user.pk)
			):
			return

		async_to_sync(self.channel_layer.group_send)(
			self.room_group_name,
			{
				'type':'chat_message',
				'author_id':user.pk,
				'content':message,
				'time':message_time,
			}
		)
		
		new_message = ChatMessageModel(
				channel_id = channel_id, 
				author_id = user.pk, 
				content = message, 
				time = message_time
		).save()

	def chat_message(self, event):

		user = self.scope["user"]
		if (user.is_anonymous or not user.is_authenticated):
			return

		channel_id: int = int(self.scope['url_route']['kwargs']['chat_id'])

		if ChatMemberModel.objects.filter(member_id=user.pk, channel_id=channel_id).count() != 1:
			return

		self.send(text_data=json.dumps({
			'type':'chat',
			'author_id':event['author_id'],
			'content':event['content'],
			'time': event['time'],
		}))
